@@ -40,15 +40,12 @@
 #include <sys/wait.h>
 #include <unistd.h>
 #include <sys/time.h>
+#include <fcntl.h>
 
 #ifdef WITH_INOTIFY
 # include <sys/inotify.h>
 #endif
 
-#ifdef HAVE_SYS_FCNTL_H
-# include <sys/fcntl.h>
-#endif
-
 #include "cronie_common.h"
 #include "funcs.h"
 #include "globals.h"
@@ -68,10 +65,12 @@ find_jobs(int, cron_db *, int, int, long),
 set_time(int),
 cron_sleep(int, cron_db *),
 sigchld_handler(int),
-sighup_handler(int),
-sigchld_reaper(void), sigintterm_handler(int), parse_args(int c, char *v[]);
+sighup_handler(int ATTRIBUTE_UNUSED),
+sigurg_handler(int ATTRIBUTE_UNUSED),
+sigchld_reaper(void),
+sigintterm_handler(int ATTRIBUTE_UNUSED), parse_args(int c, char *v[]);
 
-static volatile sig_atomic_t got_sighup, got_sigchld, got_sigintterm;
+static volatile sig_atomic_t got_sighup, got_sigchld, got_sigintterm, got_sigurg;
 static int timeRunning, virtualTime, clockTime;
 static long GMToff;
 static int DisableInotify;
@@ -153,8 +152,9 @@ void set_cron_watched(int fd) {
 #endif
 
 static void handle_signals(cron_db * database) {
-	if (got_sighup) {
+	if (got_sighup || got_sigurg) {
 		got_sighup = 0;
+		got_sigurg = 0;
 #if defined WITH_INOTIFY
 		/* watches must be reinstated on reload */
 		if (inotify_enabled && (EnableClustering != 1)) {
@@ -181,13 +181,16 @@ static void usage(void) {
 	fprintf(stderr, "\n");
 	fprintf(stderr, "Options:\n");
 	fprintf(stderr, " -h         print this message \n");
-	fprintf(stderr, " -i         deamon runs without inotify support\n");
+	fprintf(stderr, " -i         daemon runs without inotify support\n");
 	fprintf(stderr, " -m <comm>  off, or specify preferred client for sending mails\n");
 	fprintf(stderr, " -n         run in foreground\n");
+	fprintf(stderr, " -f         run in foreground, the same as -n\n");
 	fprintf(stderr, " -p         permit any crontab\n");
-	fprintf(stderr, " -P         use PATH=\"%s\"\n", _PATH_DEFPATH);
+	fprintf(stderr, " -P         inherit PATH from environment instead of using default value");
+	fprintf(stderr, "            of \"%s\"\n", _PATH_STDPATH);
 	fprintf(stderr, " -c         enable clustering support\n");
 	fprintf(stderr, " -s         log into syslog instead of sending mails\n");
+	fprintf(stderr, " -V         print version and exit\n");
 	fprintf(stderr, " -x <flag>  print debug information\n");
 	fprintf(stderr, "\n");
 	fprintf(stderr, "Debugging flags are: ");
@@ -242,13 +245,15 @@ int main(int argc, char *argv[]) {
 	sact.sa_handler = sigintterm_handler;
 	(void) sigaction(SIGINT, &sact, NULL);
 	(void) sigaction(SIGTERM, &sact, NULL);
+	sact.sa_handler = sigurg_handler;
+	(void) sigaction(SIGURG, &sact, NULL);
 
 	acquire_daemonlock(0);
 	set_cron_uid();
 	check_spool_dir();
 
 	if (ChangePath) {
-		if (setenv("PATH", _PATH_DEFPATH, 1) < 0) {
+		if (setenv("PATH", _PATH_STDPATH, 1) < 0) {
 			log_it("CRON", pid, "DEATH", "can't setenv PATH",
 				errno);
 			exit(1);
@@ -260,8 +265,8 @@ int main(int argc, char *argv[]) {
 	 */
 	setlocale(LC_ALL, "");	/* set locale to system defaults or to
 							 * that specified by any  LC_* env vars */
-	if ((cs = nl_langinfo(CODESET)) != 0L)
-		strncpy(cron_default_mail_charset, cs, MAX_ENVSTR);
+	if ((cs = nl_langinfo(CODESET)) != NULL)
+		strncpy(cron_default_mail_charset, cs, MAX_ENVSTR-1);
 	else
 		strcpy(cron_default_mail_charset, "US-ASCII");
 
@@ -288,7 +293,6 @@ int main(int argc, char *argv[]) {
 				if (fd != STDERR)
 					(void) close(fd);
 			}
-			log_it("CRON", getpid(), "STARTUP", PACKAGE_VERSION, 0);
 			break;
 		default:
 			/* parent process should just die */
@@ -296,6 +300,8 @@ int main(int argc, char *argv[]) {
 		}
 	}
 
+	log_it("CRON", getpid(), "STARTUP", PACKAGE_VERSION, 0);
+
 	if (!SyslogOutput && MailCmd[0] == '\0' && access("/usr/sbin/sendmail", X_OK) != 0) {
 		SyslogOutput=1;
 		log_it("CRON", pid, "INFO","Syslog will be used instead of sendmail.", 0);
@@ -306,17 +312,12 @@ int main(int argc, char *argv[]) {
 	/* obtain a random scaling factor for RANDOM_DELAY */
 	if (gettimeofday(&tv, &tz) != 0)
 		tv.tv_usec = 0;
-	srandom(pid + tv.tv_usec);
-	RandomScale = random() / (double)RAND_MAX;
+	srandom((unsigned int)(pid + tv.tv_usec));
+	RandomScale = (double)random() / (double)(1lu << 31);
 	snprintf(buf, sizeof(buf), "RANDOM_DELAY will be scaled with factor %d%% if used.", (int)(RandomScale*100));
 	log_it("CRON", pid, "INFO", buf, 0);
 
 	acquire_daemonlock(0);
-	database.head = NULL;
-	database.tail = NULL;
-	database.mtime = (time_t) 0;
-
-	load_database(&database);
 
 	fd = -1;
 #if defined WITH_INOTIFY
@@ -334,6 +335,12 @@ int main(int argc, char *argv[]) {
 	}
 #endif
 
+	database.head = NULL;
+	database.tail = NULL;
+	database.mtime = (time_t) 0;
+
+	load_database(&database);
+
 	set_time(TRUE);
 	run_reboot_jobs(&database);
 	timeRunning = virtualTime = clockTime;
@@ -486,6 +493,8 @@ int main(int argc, char *argv[]) {
 		log_it("CRON", pid, "INFO", "Inotify close failed", errno);
 #endif
 
+	log_it("CRON", pid, "INFO", "Shutting down", 0);
+
 	(void) unlink(_PATH_CRON_PID);
 
 	return 0;
@@ -525,7 +534,6 @@ static void find_jobs(int vtime, cron_db * db, int doWild, int doNonWild, long v
 	int minute, hour, dom, month, dow;
 	user *u;
 	entry *e;
-	const char *uname;
 
 	/* The support for the job-specific timezones is not perfect. There will
 	 * be jobs missed or run twice during the DST change in the job timezone.
@@ -554,48 +562,38 @@ static void find_jobs(int vtime, cron_db * db, int doWild, int doNonWild, long v
 
 	orig_tz = getenv("TZ");
 
-		/* the dom/dow situation is odd.  '* * 1,15 * Sun' will run on the
-		 * first and fifteenth AND every Sunday;  '* * * * Sun' will run *only*
-		 * on Sundays;  '* * 1,15 * *' will run *only* the 1st and 15th.  this
-		 * is why we keep 'e->dow_star' and 'e->dom_star'.  yes, it's bizarre.
-		 * like many bizarre things, it's the standard.
-		 */
-		for (u = db->head; u != NULL; u = u->next) {
+	/* the dom/dow situation is odd.  '* * 1,15 * Sun' will run on the
+	 * first and fifteenth AND every Sunday;  '* * * * Sun' will run *only*
+	 * on Sundays;  '* * 1,15 * *' will run *only* the 1st and 15th.  this
+	 * is why we keep 'e->dow_star' and 'e->dom_star'.  yes, it's bizarre.
+	 * like many bizarre things, it's the standard.
+	 */
+	for (u = db->head; u != NULL; u = u->next) {
 		for (e = u->crontab; e != NULL; e = e->next) {
-			Debug(DSCH | DEXT, ("user [%s:%ld:%ld:...] cmd=\"%s\"\n",
-					e->pwd->pw_name, (long) e->pwd->pw_uid,
-					(long) e->pwd->pw_gid, e->cmd));
-				uname = e->pwd->pw_name;
-			/* check if user exists in time of job is being run f.e. ldap */
-			if (getpwnam(uname) != NULL) {
-				time_t virtualSecond = (vtime - e->delay) * SECONDS_PER_MINUTE;
-				time_t virtualGMTSecond = virtualSecond - vGMToff;
-				job_tz = env_get("CRON_TZ", e->envp);
-				maketime(job_tz, orig_tz);
-				/* here we test whether time is NOW */
-				if (bit_test(e->minute, minute) &&
-					bit_test(e->hour, hour) &&
-					bit_test(e->month, month) &&
-					(((e->flags & DOM_STAR) || (e->flags & DOW_STAR))
-						? (bit_test(e->dow, dow) && bit_test(e->dom, dom))
-						: (bit_test(e->dow, dow) || bit_test(e->dom, dom))
-					)
-					) {
-					if (job_tz != NULL && vGMToff != GMToff)
-						/* do not try to run the jobs from different timezones
-						 * during the DST switch of the default timezone.
-						 */
-						continue;
+			time_t virtualSecond = (time_t)(vtime - e->delay) * (time_t)SECONDS_PER_MINUTE;
+			time_t virtualGMTSecond = virtualSecond - vGMToff;
+			job_tz = env_get("CRON_TZ", e->envp);
+			maketime(job_tz, orig_tz);
 
-					if ((doNonWild &&
-							!(e->flags & (MIN_STAR | HR_STAR))) ||
-						(doWild && (e->flags & (MIN_STAR | HR_STAR))))
-						job_add(e, u);	/*will add job, if it isn't in queue already for NOW. */
-				}
-			}
-			else {
-				log_it(uname, getpid(), "ERROR", "getpwnam() failed",errno);
-				Debug(DSCH | DEXT, ("%s:%d pid=%d time=%ld getpwnam(%s) failed errno=%d error=%s\n",__FILE__,__LINE__,getpid(),time(NULL),uname,errno,strerror(errno)));
+			/* here we test whether time is NOW */
+			if (bit_test(e->minute, minute) &&
+				bit_test(e->hour, hour) &&
+				bit_test(e->month, month) &&
+				(((e->flags & DOM_STAR) || (e->flags & DOW_STAR))
+					? (bit_test(e->dow, dow) && bit_test(e->dom, dom))
+						: (bit_test(e->dow, dow) || bit_test(e->dom, dom))
+				)
+			) {
+				if (job_tz != NULL && vGMToff != GMToff)
+					/* do not try to run the jobs from different timezones
+					 * during the DST switch of the default timezone.
+					 */
+					continue;
+
+				if ((doNonWild &&
+						!(e->flags & (MIN_STAR | HR_STAR))) ||
+					(doWild && (e->flags & (MIN_STAR | HR_STAR))))
+					job_add(e, u);	/*will add job, if it isn't in queue already for NOW. */
 			}
 		}
 	}
@@ -623,7 +621,7 @@ static void set_time(int initialize) {
 		GMToff = get_gmtoff(&StartTime, &tm);
 		Debug(DSCH, ("[%ld] GMToff=%ld\n", (long) getpid(), (long) GMToff));
 	}
-	clockTime = (StartTime + GMToff) / (time_t) SECONDS_PER_MINUTE;
+	clockTime = (int)((StartTime + GMToff) / (time_t) SECONDS_PER_MINUTE);
 }
 
 /*
@@ -658,18 +656,22 @@ static void cron_sleep(int target, cron_db * db) {
 	}
 }
 
-static void sighup_handler(int x) {
+static void sighup_handler(int x ATTRIBUTE_UNUSED) {
 	got_sighup = 1;
 }
 
-static void sigchld_handler(int x) {
+static void sigchld_handler(int x ATTRIBUTE_UNUSED) {
 	got_sigchld = 1;
 }
 
-static void sigintterm_handler(int x) {
+static void sigintterm_handler(int x ATTRIBUTE_UNUSED) {
 	got_sigintterm = 1;
 }
 
+static void sigurg_handler(int x ATTRIBUTE_UNUSED) {
+	got_sigurg = 1;
+}
+
 static void sigchld_reaper(void) {
 	WAIT_T waiter;
 	PID_T pid;
@@ -681,15 +683,15 @@ static void sigchld_reaper(void) {
 			if (errno == EINTR)
 				continue;
 			Debug(DPROC, ("[%ld] sigchld...no children\n", (long) getpid()));
-				break;
+			break;
 		case 0:
 			Debug(DPROC, ("[%ld] sigchld...no dead kids\n", (long) getpid()));
-				break;
+			break;
 		default:
 			Debug(DPROC,
 				("[%ld] sigchld...pid #%ld died, stat=%d\n",
 					(long) getpid(), (long) pid, WEXITSTATUS(waiter)));
-				break;
+			break;
 		}
 	} while (pid > 0);
 }
@@ -697,13 +699,14 @@ static void sigchld_reaper(void) {
 static void parse_args(int argc, char *argv[]) {
 	int argch;
 
-	while (-1 != (argch = getopt(argc, argv, "hnpsiPx:m:c"))) {
+	while (-1 != (argch = getopt(argc, argv, "hnfpsiPx:m:cV"))) {
 		switch (argch) {
 			case 'x':
 				if (!set_debug_flags(optarg))
 					usage();
 				break;
 			case 'n':
+			case 'f':
 				NoFork = 1;
 				break;
 			case 'p':
@@ -724,6 +727,9 @@ static void parse_args(int argc, char *argv[]) {
 			case 'c':
 				EnableClustering = 1;
 				break;
+			case 'V':
+				puts(PACKAGE_STRING);
+				exit(EXIT_SUCCESS);
 			case 'h':
 			default:
 				usage();
