@@ -30,11 +30,13 @@
 #include <string.h>
 #include <sys/wait.h>
 #include <unistd.h>
+#include <dirent.h>
 
 #include "externs.h"
 #include "funcs.h"
 #include "globals.h"
 #include "structs.h"
+#include "cronie_common.h"
 
 #ifndef isascii
 # define isascii(c)	((unsigned)(c)<=0177)
@@ -46,7 +48,7 @@ static int safe_p(const char *, const char *);
 void do_command(entry * e, user * u) {
 	pid_t pid = getpid();
 	int ev;
-	char **jobenv = 0L;
+	char **jobenv = NULL;
 
 	Debug(DPROC, ("[%ld] do_command(%s, (%s,%ld,%ld))\n",
 			(long) pid, e->cmd, u->name,
@@ -89,8 +91,11 @@ void do_command(entry * e, user * u) {
 static int child_process(entry * e, char **jobenv) {
 	int stdin_pipe[2], stdout_pipe[2];
 	char *input_data, *usernm, *mailto, *mailfrom;
+	char mailto_expanded[MAX_EMAILSTR];
+	char mailfrom_expanded[MAX_EMAILSTR];
 	int children = 0;
 	pid_t pid = getpid();
+	pid_t jobpid = -1;
 	struct sigaction sa;
 
 	/* Ignore SIGPIPE as we will be writing to pipes and do not want to terminate
@@ -110,14 +115,14 @@ static int child_process(entry * e, char **jobenv) {
 
 	Debug(DPROC, ("[%ld] child_process('%s')\n", (long) getpid(), e->cmd));
 #ifdef CAPITALIZE_FOR_PS
-		/* mark ourselves as different to PS command watchers by upshifting
-		 * our program name.  This has no effect on some kernels.
-		 */
-		/*local */  {
+	/* mark ourselves as different to PS command watchers by upshifting
+	 * our program name.  This has no effect on some kernels.
+	 */
+	/*local */  {
 		char *pch;
 
 		for (pch = ProgramName; *pch; pch++)
-			*pch = MkUpper(*pch);
+			*pch = (char)MkUpper(*pch);
 	}
 #endif /* CAPITALIZE_FOR_PS */
 
@@ -126,6 +131,24 @@ static int child_process(entry * e, char **jobenv) {
 	usernm = e->pwd->pw_name;
 	mailto = env_get("MAILTO", jobenv);
 	mailfrom = env_get("MAILFROM", e->envp);
+	
+	if (mailto != NULL) {
+		if (expand_envvar(mailto, mailto_expanded, sizeof(mailto_expanded))) {
+			mailto = mailto_expanded;
+		}
+		else {
+			log_it("CRON", pid, "WARNING", "The environment variable 'MAILTO' could not be expanded. The non-expanded value will be used." , 0);
+		}
+	}
+	
+	if (mailfrom != NULL) {
+		if (expand_envvar(mailfrom, mailfrom_expanded, sizeof(mailfrom_expanded))) {
+			mailfrom = mailfrom_expanded;
+		}
+		else {
+			log_it("CRON", pid, "WARNING", "The environment variable 'MAILFROM' could not be expanded. The non-expanded value will be used." , 0);
+		}
+	}
 
 	/* create some pipes to talk to our future child
 	 */
@@ -156,10 +179,10 @@ static int child_process(entry * e, char **jobenv) {
 		for (input_data = p = e->cmd;
 			(ch = *input_data) != '\0'; input_data++, p++) {
 			if (p != input_data)
-				*p = ch;
+				*p = (char)ch;
 			if (escaped) {
 				if (ch == '%')
-					*--p = ch;
+					*--p = (char)ch;
 				escaped = FALSE;
 				continue;
 			}
@@ -178,7 +201,7 @@ static int child_process(entry * e, char **jobenv) {
 
 	/* fork again, this time so we can exec the user's command.
 	 */
-	switch (fork()) {
+	switch (jobpid = fork()) {
 	case -1:
 		log_it("CRON", pid, "CAN'T FORK", "child_process", errno);
 		return ERROR_EXIT;
@@ -194,6 +217,9 @@ static int child_process(entry * e, char **jobenv) {
 		if ((e->flags & DONT_LOG) == 0) {
 			char *x = mkprints((u_char *) e->cmd, strlen(e->cmd));
 
+			if (x == NULL) /* out of memory, better exit */
+				_exit(ERROR_EXIT);
+
 			log_it(usernm, getpid(), "CMD", x, 0);
 			free(x);
 		}
@@ -238,11 +264,27 @@ static int child_process(entry * e, char **jobenv) {
 		 */
 		{
 			char *shell = env_get("SHELL", jobenv);
-			int fd, fdmax = getdtablesize();
+			int fd, fdmax = TMIN(getdtablesize(), MAX_CLOSE_FD);
+			DIR *dir;
+			struct dirent *dent;
 
-			/* close all unwanted open file descriptors */
-			for(fd = STDERR + 1; fd < fdmax; fd++) {
-				close(fd);
+			/*
+			 * if /proc is mounted, we can optimize what fd can be closed,
+			 * but if it isn't available, fall back to the previous behavior.
+			 */
+			if ((dir = opendir("/proc/self/fd")) != NULL) {
+				while ((dent = readdir(dir)) != NULL) {
+					if (!strcmp(dent->d_name, ".") || !strcmp(dent->d_name, ".."))
+						continue;
+					fd = atoi(dent->d_name);
+					if (fd > STDERR_FILENO)
+						close(fd);
+				}
+			} else {
+				/* close all unwanted open file descriptors */
+				for (fd = STDERR + 1; fd < fdmax; fd++) {
+					close(fd);
+				}
 			}
 
 #if DEBUGGING
@@ -376,7 +418,7 @@ static int child_process(entry * e, char **jobenv) {
 			int bufidx = 0;
 			if (SyslogOutput) {
 				if (ch != '\n')
-					logbuf[bufidx++] = ch;
+					logbuf[bufidx++] = (char)ch;
 			}
 #endif
 
@@ -418,7 +460,7 @@ static int child_process(entry * e, char **jobenv) {
 			if (mailto && safe_p(usernm, mailto)
 				&& strncmp(MailCmd,"off",3) && !SyslogOutput) {
 				char **env;
-				char mailcmd[MAX_COMMAND];
+				char mailcmd[MAX_COMMAND+1]; /* +1 for terminator */
 				char hostname[MAXHOSTNAMELEN];
 				char *content_type = env_get("CONTENT_TYPE", jobenv),
 					*content_transfer_encoding =
@@ -427,16 +469,22 @@ static int child_process(entry * e, char **jobenv) {
 				gethostname(hostname, MAXHOSTNAMELEN);
 
 				if (MailCmd[0] == '\0') {
-					if (snprintf(mailcmd, sizeof mailcmd, MAILFMT, MAILARG, mailfrom)
-						>= sizeof mailcmd) {
+					int len;
+
+					len = snprintf(mailcmd, sizeof mailcmd, MAILFMT, MAILARG);
+					if (len < 0) {
+						fprintf(stderr, "mailcmd snprintf failed\n");
+						(void) _exit(ERROR_EXIT);
+					}
+					if (sizeof mailcmd <= (size_t) len) {
 						fprintf(stderr, "mailcmd too long\n");
 						(void) _exit(ERROR_EXIT);
 					}
 				}
 				else {
-					strncpy(mailcmd, MailCmd, MAX_COMMAND);
+					strncpy(mailcmd, MailCmd, MAX_COMMAND+1);
 				}
-				if (!(mail = cron_popen(mailcmd, "w", e->pwd))) {
+				if (!(mail = cron_popen(mailcmd, "w", e->pwd, jobenv))) {
 					perror(mailcmd);
 					(void) _exit(ERROR_EXIT);
 				}
@@ -449,7 +497,8 @@ static int child_process(entry * e, char **jobenv) {
 #ifdef MAIL_DATE
 				fprintf(mail, "Date: %s\n", arpadate(&StartTime));
 #endif /*MAIL_DATE */
-				if (content_type == 0L) {
+				fprintf(mail, "MIME-Version: 1.0\n");
+				if (content_type == NULL) {
 					fprintf(mail, "Content-Type: text/plain; charset=%s\n",
 						cron_default_mail_charset);
 				}
@@ -460,17 +509,20 @@ static int child_process(entry * e, char **jobenv) {
 					char *nl = content_type;
 					size_t ctlen = strlen(content_type);
 					while ((*nl != '\0')
-						&& ((nl = strchr(nl, '\n')) != 0L)
+						&& ((nl = strchr(nl, '\n')) != NULL)
 						&& (nl < (content_type + ctlen))
 						)
 						*nl = ' ';
 					fprintf(mail, "Content-Type: %s\n", content_type);
 				}
-				if (content_transfer_encoding != 0L) {
+				if (content_transfer_encoding == NULL) {
+					fprintf(mail, "Content-Transfer-Encoding: 8bit\n");
+				}
+				else {
 					char *nl = content_transfer_encoding;
 					size_t ctlen = strlen(content_transfer_encoding);
 					while ((*nl != '\0')
-						&& ((nl = strchr(nl, '\n')) != 0L)
+						&& ((nl = strchr(nl, '\n')) != NULL)
 						&& (nl < (content_transfer_encoding + ctlen))
 						)
 						*nl = ' ';
@@ -499,12 +551,14 @@ static int child_process(entry * e, char **jobenv) {
 			 */
 
 			while (EOF != (ch = getc(in))) {
+				if (ch == '\r')
+					continue;
 				bytes++;
 				if (mail)
 					putc(ch, mail);
 #if defined(SYSLOG)
 				if (SyslogOutput) {
-					logbuf[bufidx++] = ch;
+					logbuf[bufidx++] = (char)ch;
 					if ((ch == '\n') || (bufidx == sizeof(logbuf)-1)) {
 						if (ch == '\n')
 							logbuf[bufidx-1] = '\0';
@@ -516,10 +570,35 @@ static int child_process(entry * e, char **jobenv) {
 				}
 #endif
 			}
-			/* only close pipe if we opened it -- i.e., we're
+			/* if -n option was specified, abort the sending
+			 * now when we read all of the command output
+			 * and thus can wait for it's exit status
+			 */
+			if (mail && e->flags & MAIL_WHEN_ERR) {
+				int jobstatus = -1;
+				if (jobpid > 0) {
+					while (waitpid(jobpid, &jobstatus, WNOHANG) == -1) {
+						if (errno == EINTR) continue;
+						log_it("CRON", getpid(), "error", "invalid job pid", errno);
+						break;
+					}
+				} else {
+					log_it("CRON", getpid(), "error", "invalid job pid", 0);
+				}
+
+				/* if everything went well, -n is set, and we have mail,
+				 * we won't be mailing – so shoot the messenger!
+				 */
+				if (WIFEXITED(jobstatus) && WEXITSTATUS(jobstatus) == EXIT_SUCCESS) {
+					Debug(DPROC, ("[%ld] aborting pipe to mail\n", (long)getpid()));
+					status = cron_pabort(mail);
+					mail = NULL;
+				}
+			}
+
+			/* only close pipe if we opened it -- i.e., we're (still)
 			 * mailing...
 			 */
-
 			if (mail) {
 				Debug(DPROC, ("[%ld] closing pipe to mail\n", (long) getpid()));
 					/* Note: the pclose will probably see
@@ -567,18 +646,24 @@ static int child_process(entry * e, char **jobenv) {
 
 		Debug(DPROC, ("[%ld] waiting for grandchild #%d to finish\n",
 				(long) getpid(), children));
-			while ((child = wait(&waiter)) < OK && errno == EINTR) ;
+		while ((child = wait(&waiter)) < OK && errno == EINTR) ;
 		if (child < OK) {
 			Debug(DPROC,
 				("[%ld] no more grandchildren--mail written?\n",
 					(long) getpid()));
-				break;
+			break;
 		}
 		Debug(DPROC, ("[%ld] grandchild #%ld finished, status=%04x",
 				(long) getpid(), (long) child, WEXITSTATUS(waiter)));
 			if (WIFSIGNALED(waiter) && WCOREDUMP(waiter))
-			Debug(DPROC, (", dumped core"));
-				Debug(DPROC, ("\n"));
+				Debug(DPROC, (", dumped core"));
+			Debug(DPROC, ("\n"));
+	}
+	if ((e->flags & DONT_LOG) == 0) {
+		char *x = mkprints((u_char *) e->cmd, strlen(e->cmd));
+
+		log_it(usernm, getpid(), "CMDEND", x ? x : "**Unknown command**" , 0);
+		free(x);
 	}
 	return OK_EXIT;
 }
